package jdk.internal.instrumentation;

import java.util.Iterator;
import jdk.internal.org.objectweb.asm.Type;
import jdk.internal.org.objectweb.asm.tree.MethodNode;
import jdk.internal.org.objectweb.asm.MethodVisitor;
import jdk.internal.org.objectweb.asm.ClassReader;
import java.lang.reflect.Method;
import java.util.List;
import jdk.internal.org.objectweb.asm.tree.ClassNode;
import jdk.internal.org.objectweb.asm.ClassVisitor;

final class Inliner extends ClassVisitor
{
    private final String instrumentationClassName;
    private final Logger logger;
    private final ClassNode targetClassNode;
    private final List<Method> instrumentationMethods;
    private final MaxLocalsTracker maxLocalsTracker;
    
    Inliner(int api, ClassVisitor classVisitor, String instrumentationClassName, ClassReader classReader, List<Method> instrumentationMethods, MaxLocalsTracker maxLocalsTracker, Logger logger) {
        super(api, classVisitor);
        this.instrumentationClassName = instrumentationClassName;
        this.instrumentationMethods = instrumentationMethods;
        this.maxLocalsTracker = maxLocalsTracker;
        this.logger = logger;
        ClassNode classNode = new ClassNode(327680);
        classReader.accept(classNode, 8);
        this.targetClassNode = classNode;
    }
    
    @Override
    public MethodVisitor visitMethod(int access, String s, String s2, String signature, String[] exceptions) {
        MethodVisitor visitMethod = super.visitMethod(access, s, s2, signature, exceptions);
        if (this.isInstrumentationMethod(s, s2)) {
            MethodNode targetMethodNode = this.findTargetMethodNode(s, s2);
            if (targetMethodNode == null) {
                throw new IllegalArgumentException("Could not find the method to instrument in the target class");
            }
            if ((targetMethodNode.access & 0x100) == 0x1) {
                throw new IllegalArgumentException("Cannot instrument native methods: " + this.targetClassNode.name + "." + targetMethodNode.name + targetMethodNode.desc);
            }
            this.logger.trace("Inliner processing method " + s + s2);
            return new MethodCallInliner(access, s2, visitMethod, targetMethodNode, this.instrumentationClassName, this.maxLocalsTracker.getMaxLocals(s, s2), this.logger);
        }
        else {
            return visitMethod;
        }
    }
    
    private boolean isInstrumentationMethod(String anObject, String anObject2) {
        for (Method method : this.instrumentationMethods) {
            if (method.getName().equals(anObject) && Type.getMethodDescriptor(method).equals(anObject2)) {
                return true;
            }
            continue;
        }
        return false;
    }
    
    private MethodNode findTargetMethodNode(String s, String s2) {
        for (MethodNode methodNode : this.targetClassNode.methods) {
            if (methodNode.desc.equals(s2) && methodNode.name.equals(s)) {
                return methodNode;
            }
            continue;
        }
        throw new IllegalArgumentException("could not find MethodNode for " + s + s2);
    }
}
